/*
 * Code for controlling homemade Predator style lasergun
 * 
 * Requires:
 * 2 5V Relays
 * 1 red LED + 220 Ohm resistor
 * 1 green LED + 220 Ohm resistor
 * 2 single channel switches
 * 1 push button
 * 3 10K Ohm resistors
 * 3 HLM1230 Red laser modules
 * 1 Large main laser
 * 
 * By Joep Eijkemans: www.joepeijkemans.nl
 * 
 */
const int trigger = 3; //pin for the trigger button
    
const int Laser = 4; //pins for main laser and predator sights Relays
const int predatorSights = 5;

const int aimButton = 6; //pins for predator sights and safe switches
const int safeButton = 7;

const int green = 8; //pins for green and red LEDs
const int red = 9;

int buttonStateSafe = 0; //button states for safe and predator sights switches
int buttonStateAim = 0;

int lastButtonStateSafe = 0; //previous button states for safe and predator sights switches for state change detection    
int lastButtonStateAim = 0; 
bool safe; //bool for whether or not safe state is on


void setup() {
  pinMode(trigger, INPUT); //trigger button = input
  pinMode(safeButton, INPUT); //safe switch = input
  pinMode(aimButton, INPUT); // predator sights switch = input
  
  pinMode(green,OUTPUT); //green LED = output
  pinMode(red, OUTPUT); //red LED = output
  pinMode(Laser, OUTPUT); //laser Relay = output
  pinMode(predatorSights, OUTPUT); //predator sights Relay = output
  Serial.begin(9600); //initialize serial monitor


  digitalWrite(Laser, HIGH); //turn laser off by default

  if(digitalRead(aimButton)==HIGH){ //if predator sights switch is on
    Serial.println("aim laser on"); //print feedback
    digitalWrite(predatorSights, LOW); //activate predator sights
  }  else {
    Serial.println("aim laser off"); //print feedback
    digitalWrite(predatorSights, HIGH); //deactivate predator sights
  }

  if(digitalRead(safeButton) == LOW){ //if safe switch is on
    Serial.println("Safe on"); //print feedback
    digitalWrite(green, HIGH); //enable green LED
    digitalWrite(red, LOW); //disable red LED
    safe = true; //set safe to true
    //turning the laser off here is unnecassary because it is turned of by default already
  }else{
    Serial.println("Safe off"); //print feedback
    digitalWrite(green, LOW); //disable green LED
    digitalWrite(red, HIGH); //enable red LED
    safe = false; //set safe to false
  }
  delay(100); //small delay before we start
}


void loop() {
buttonStateSafe = digitalRead(safeButton); //read the states of the safe and aim switches
buttonStateAim = digitalRead(aimButton);

if(buttonStateSafe != lastButtonStateSafe){ //state change detection for safe button, this ensures we only have to do things once rather than every frame
  if(buttonStateSafe == HIGH){ //if turned off
    Serial.println("Safe off"); //print feedback
    safe = false; //set safe to false
    digitalWrite(green, LOW); //switch green to red LED 
    digitalWrite(red, HIGH);
  }
  else //if turned on
  {
    Serial.println("Safe on"); //print feedback
    digitalWrite(Laser, HIGH); //deactivate laser so that if you switch safe on whilst pressing trigger, the laser still shuts off
    safe = true; //set safe to true
    digitalWrite(green, HIGH); //switch red to green LED
    digitalWrite(red, LOW);
  }
}
if(buttonStateAim != lastButtonStateAim){ //state change detection for the predator sights switch
  if(buttonStateAim == HIGH){ //if turned on
    Serial.println("aim laser on"); //print feedback
    digitalWrite(predatorSights, LOW); //turn on predator sights
  }
  else //if turned off
  { 
    Serial.println("aim laser off"); //print feedback
    digitalWrite(predatorSights, HIGH); //turn off predator sights
  }
}

if(!safe){ //if safe is off
  if(digitalRead(trigger)==HIGH){ //turn the main laser on if the trigger is pressed
    digitalWrite(Laser, LOW);
  }else{ 
    digitalWrite(Laser, HIGH); //turn the main laser off if the trigger isn't pressed
  }
}
lastButtonStateSafe = buttonStateSafe; //safe previous state of safe switch for state change detection
lastButtonStateAim = buttonStateAim; //safe previous state of predator sights switch for state change detection
delay(200); //delay to ensure constant framerate and prevent contact bouncing. Could be lower but this framerate is sufficient
}
